%ifarch x86_64
%global with_crocus 1
%global with_iris   1
%global with_vmware 1
%global with_xa     1
%global platform_dri ,i915,i965
%global platform_vulkan ,intel
%endif

%ifarch aarch64
%global with_freedreno 1
%global with_kmsro     1
%global with_panfrost  1
%global with_tegra     1
%global with_xa        1
%global platform_vulkan ,broadcom,freedreno,panfrost
%endif

%ifarch %{valgrind_arches}
%bcond_without valgrind
%else
%bcond_with valgrind
%endif

Name:           mesa
Summary:        Mesa graphics libraries
Version:        23.1.4
Release:        7%{?dist}
License:        MIT
URL:            http://www.mesa3d.org
Source0:        https://mesa.freedesktop.org/archive/%{name}-%{version}.tar.xz
Source1:        Mesa-MLAA-License-Clarification-Email.txt

# fix build error in llvm 17
Patch0001: 	0001-llvmpipe-only-include-old-Transform-includes-when-ne.patch

# add an implementation with llvm orcjit and support loongarch64 orcjit
Patch0002:      0001-llvmpipe-add-an-implementation-with-llvm-orcjit.patch
Patch0003:      0001-llvmpipe-add-loongarch64-basic-support.patch
Patch0004:      0002-llvmpipe-add-loongarch64-orcjit.patch

BuildRequires:  meson >= 0.45, gcc, gcc-c++, kernel-headers, llvm-devel >= 7.0.0
BuildRequires:  pkgconfig(libdrm) >= 2.4.97, pkgconfig(expat), pkgconfig(zlib) >= 1.2.3
BuildRequires:  pkgconfig(libselinux), pkgconfig(wayland-scanner), pkgconfig(wayland-protocols) >= 1.8
BuildRequires:  pkgconfig(wayland-client) >= 1.11, pkgconfig(wayland-server) >= 1.11, pkgconfig(wayland-egl-backend) >= 3
BuildRequires:  pkgconfig(x11), pkgconfig(xext), pkgconfig(xdamage) >= 1.1, pkgconfig(xfixes), pkgconfig(xcb-glx) >= 1.8.1
BuildRequires:  pkgconfig(xxf86vm), pkgconfig(xcb), pkgconfig(x11-xcb), pkgconfig(xcb-dri2) >= 1.8, pkgconfig(xcb-dri3)
BuildRequires:  pkgconfig(xcb-present), pkgconfig(xcb-sync), pkgconfig(xshmfence) >= 1.1, pkgconfig(dri2proto) >= 2.8
BuildRequires:  pkgconfig(glproto) >= 1.4.14, pkgconfig(xcb-xfixes), pkgconfig(xcb-randr), pkgconfig(xrandr) >= 1.3
BuildRequires:  pkgconfig(vdpau) >= 1.1, pkgconfig(libva) >= 0.38.0
BuildRequires:  pkgconfig(libelf), pkgconfig(libglvnd) >= 1.3.2
BuildRequires:  flex, bison, gettext, libunwind-devel
%if %{with valgrind}
BuildRequires:  pkgconfig(valgrind)
%endif
BuildRequires:  python3-devel, python3-mako
BuildRequires:  vulkan-headers, pkgconfig(vulkan)
BuildRequires:  glslang

%description
The Mesa project began as an open-source implementation of the OpenGL specification - a system
for rendering interactive 3D graphics. Over the years the project has grown to implement more
graphics APIs, including OpenGL ES, OpenCL, OpenMAX, VDPAU, VA-API, Vulkan and EGL.A variety of
device drivers allows the Mesa libraries to be used in many different environments ranging from
software emulation to complete hardware acceleration for modern GPUs. Mesa ties into several other
open-source projects: the Direct Rendering Infrastructure, X.org, and Wayland to provide OpenGL
support on Linux, FreeBSD, and other operating systems.



%package filesystem
Summary:        Mesa driver filesystem
Provides:       mesa-dri-filesystem = %{version}-%{release}

%description filesystem
%{summary}.

%package libGL
Summary:        Mesa libGL runtime libraries
Requires:       %{name}-libglapi = %{version}-%{release}
Requires:       libglvnd-glx >= 1.3.2

%description libGL
%{summary}.

%package libGL-devel
Summary:        Mesa libGL development package
Requires:       %{name}-libGL = %{version}-%{release}
Requires:       libglvnd-devel >= 1.3.2
Provides:       libGL-devel
Recommends:     gl-manpages

%description libGL-devel
%{summary}.

%package libEGL
Summary:        Mesa libEGL runtime libraries
Requires:       libglvnd-egl >= 1.3.2

%description libEGL
%{summary}.

%package libEGL-devel
Summary:        Mesa libEGL development package
Requires:       %{name}-libEGL = %{version}-%{release}
Requires:       libglvnd-devel >= 1.3.2
Requires:       %{name}-khr-devel
Provides:       libEGL-devel

%description libEGL-devel
%{summary}.

%package dri-drivers
Summary:        Mesa-based DRI drivers
Requires:       %{name}-filesystem = %{version}-%{release}

%description dri-drivers
%{summary}.

%package        vdpau-drivers
Summary:        Mesa-based VDPAU drivers
Requires:       %{name}-filesystem = %{version}-%{release}

%description vdpau-drivers
%{summary}.

%package libOSMesa
Summary:        Mesa offscreen rendering libraries
Requires:       %{name}-libglapi = %{version}-%{release}
Provides:       libOSMesa

%description libOSMesa
%{summary}.

%package libOSMesa-devel
Summary:        Mesa offscreen rendering development package
Requires:       %{name}-libOSMesa = %{version}-%{release}

%description libOSMesa-devel
%{summary}.

%package libgbm
Summary:        Mesa gbm runtime library
Provides:       libgbm

%description libgbm
%{summary}.

%package libgbm-devel
Summary:        Mesa libgbm development package
Requires:       %{name}-libgbm = %{version}-%{release}
Provides:       libgbm-devel

%description libgbm-devel
%{summary}.

%if 0%{?with_xa}
%package libxatracker
Summary:        Mesa XA state tracker
Provides:       libxatracker
Provides:       libxatracker

%description libxatracker
%{summary}.

%package libxatracker-devel
Summary:        Mesa XA state tracker development package
Requires:       %{name}-libxatracker = %{version}-%{release}
Provides:       libxatracker-devel

%description libxatracker-devel
%{summary}.
%endif

%package libglapi
Summary:        Mesa shared glapi
Provides:       libglapi

%description libglapi
%{summary}.

%package vulkan-drivers
Summary:        Mesa Vulkan drivers
Requires:       vulkan

%description vulkan-drivers
The drivers with support for the Vulkan API.

%package vulkan-devel
Summary:        Mesa Vulkan development files
Requires:       %{name}-vulkan-drivers = %{version}-%{release}
Requires:       vulkan-devel

%description vulkan-devel
Headers for development with the Vulkan API.

%package        va-drivers
Summary:        Mesa-based VA-API video acceleration drivers
Requires:       %{name}-filesystem = %{version}-%{release}

%description va-drivers
%{summary}.

%prep
%autosetup -p1
cp %{SOURCE1} docs/

%build
%define _lto_cflags %{nil}

%meson \
  -Dplatforms=x11,wayland \
  -Ddri3=enabled \
  -Dosmesa=true \
  -Dgallium-drivers=swrast,virgl,nouveau%{?with_crocus:,crocus}%{?with_iris:,iris}%{?with_vmware:,svga},radeonsi%{?with_freedreno:,freedreno}%{?with_tegra:,tegra}%{?with_kmsro:,kmsro}%{?with_panfrost:,panfrost},zink \
  -Dgallium-vdpau=enabled \
  -Dgallium-omx=disabled \
  -Dgallium-va=enabled \
  -Dgallium-xa=%{?with_xa:enabled}%{!?with_xa:disabled} \
  -Dgallium-nine=false \
  -Dgallium-opencl=disabled \
  -Dvulkan-drivers=swrast,amd%{?platform_vulkan} \
  -Dvulkan-layers=device-select \
  -Dshared-glapi=enabled \
  -Dgles1=disabled \
  -Dgles2=enabled \
  -Dopengl=true \
  -Dgbm=enabled \
  -Dglx=dri \
  -Degl=enabled \
  -Dglvnd=true \
  -Dmicrosoft-clc=disabled \
  -Dllvm=enabled \
  -Dllvm-orcjit=true \
  -Dshared-llvm=enabled \
  -Dvalgrind=%{?with_valgrind:enabled}%{!?with_valgrind:disabled} \
  -Dbuild-tests=false \
  -Dselinux=true \
  -Dlmsensors=disabled \
  -Dandroid-libbacktrace=disabled \
  %{nil}
%meson_build

%install
%meson_install

rm -vf %{buildroot}%{_libdir}/vdpau/*.so
rm -vf %{buildroot}%{_libdir}/libGLX_mesa.so
rm -vf %{buildroot}%{_libdir}/libEGL_mesa.so
rm -vf %{buildroot}%{_libdir}/libGLES*
ln -s %{_libdir}/libGLX_mesa.so.0 %{buildroot}%{_libdir}/libGLX_system.so.0

%files filesystem
%doc docs/Mesa-MLAA-License-Clarification-Email.txt
%dir %{_libdir}/dri
%dir %{_libdir}/vdpau

%files libGL
%{_libdir}/libGLX_mesa.so.0*
%{_libdir}/libGLX_system.so.0*

%files libGL-devel
%dir %{_includedir}/GL/internal
%{_includedir}/GL/internal/dri_interface.h
%{_libdir}/pkgconfig/dri.pc
%{_libdir}/libglapi.so

%files libEGL
%{_datadir}/glvnd/egl_vendor.d/50_mesa.json
%{_libdir}/libEGL_mesa.so.0*

%files libEGL-devel
%dir %{_includedir}/EGL
%{_includedir}/EGL/eglmesaext.h
%{_includedir}/EGL/eglext_angle.h

%files libglapi
%{_libdir}/libglapi.so.0
%{_libdir}/libglapi.so.0.*

%files libOSMesa
%{_libdir}/libOSMesa.so.8*

%files libOSMesa-devel
%dir %{_includedir}/GL
%{_includedir}/GL/osmesa.h
%{_libdir}/libOSMesa.so
%{_libdir}/pkgconfig/osmesa.pc

%files libgbm
%{_libdir}/libgbm.so.1
%{_libdir}/libgbm.so.1.*

%files libgbm-devel
%{_libdir}/libgbm.so
%{_includedir}/gbm.h
%{_libdir}/pkgconfig/gbm.pc

%if 0%{?with_xa}
%files libxatracker
%{_libdir}/libxatracker.so.2
%{_libdir}/libxatracker.so.2.*

%files libxatracker-devel
%{_libdir}/libxatracker.so
%{_includedir}/xa_tracker.h
%{_includedir}/xa_composite.h
%{_includedir}/xa_context.h
%{_libdir}/pkgconfig/xatracker.pc
%endif

%files dri-drivers
%dir %{_datadir}/drirc.d
%{_datadir}/drirc.d/00-mesa-defaults.conf
%{_libdir}/dri/kms_swrast_dri.so
%{_libdir}/dri/swrast_dri.so
%{_libdir}/dri/virtio_gpu_dri.so
%{_libdir}/dri/virtio_gpu_drv_video.so
%{_libdir}/dri/radeonsi_dri.so
%ifarch x86_64
%{_libdir}/dri/crocus_dri.so
%{_libdir}/dri/iris_dri.so
%endif
%ifarch aarch64
%{_libdir}/dri/ingenic-drm_dri.so
%{_libdir}/dri/imx-drm_dri.so
%{_libdir}/dri/imx-lcdif_dri.so
%{_libdir}/dri/kirin_dri.so
%{_libdir}/dri/komeda_dri.so
%{_libdir}/dri/mali-dp_dri.so
%{_libdir}/dri/mcde_dri.so
%{_libdir}/dri/mxsfb-drm_dri.so
%{_libdir}/dri/rcar-du_dri.so
%{_libdir}/dri/stm_dri.so
%endif
%if 0%{?with_freedreno}
%{_libdir}/dri/kgsl_dri.so
%{_libdir}/dri/msm_dri.so
%endif
%if 0%{?with_tegra}
%{_libdir}/dri/tegra_dri.so
%endif
%if 0%{?with_panfrost}
%{_libdir}/dri/panfrost_dri.so
%endif
%{_libdir}/dri/nouveau_dri.so
%if 0%{?with_vmware}
%{_libdir}/dri/vmwgfx_dri.so
%endif
%if 0%{?with_kmsro}
%{_libdir}/dri/armada-drm_dri.so
%{_libdir}/dri/exynos_dri.so
%{_libdir}/dri/hx8357d_dri.so
%{_libdir}/dri/ili9225_dri.so
%{_libdir}/dri/ili9341_dri.so
%{_libdir}/dri/imx-dcss_dri.so
%{_libdir}/dri/mediatek_dri.so
%{_libdir}/dri/meson_dri.so
%{_libdir}/dri/mi0283qt_dri.so
%{_libdir}/dri/pl111_dri.so
%{_libdir}/dri/repaper_dri.so
%{_libdir}/dri/rockchip_dri.so
%{_libdir}/dri/st7586_dri.so
%{_libdir}/dri/st7735r_dri.so
%{_libdir}/dri/sun4i-drm_dri.so
%endif
%{_libdir}/dri/zink_dri.so

%files vdpau-drivers
%{_libdir}/vdpau/libvdpau_nouveau.so.1*
%{_libdir}/vdpau/libvdpau_virtio_gpu.so.1*
%{_libdir}/vdpau/libvdpau_radeonsi.so.1*

%files vulkan-drivers
%{_datadir}/drirc.d/00-radv-defaults.conf
%{_libdir}/libvulkan_lvp.so
%{_datadir}/vulkan/icd.d/lvp_icd.*.json
%{_libdir}/libVkLayer_MESA_device_select.so
%{_datadir}/vulkan/implicit_layer.d/VkLayer_MESA_device_select.json
%{_libdir}/libvulkan_radeon.so
%{_datadir}/vulkan/icd.d/radeon_icd.*.json
%ifarch x86_64
%{_libdir}/libvulkan_intel.so
%{_datadir}/vulkan/icd.d/intel_icd.*.json
%endif
%ifarch aarch64
%{_libdir}/libvulkan_broadcom.so
%{_datadir}/vulkan/icd.d/broadcom_icd.*.json
%{_libdir}/libvulkan_freedreno.so
%{_datadir}/vulkan/icd.d/freedreno_icd.*.json
%{_libdir}/libvulkan_panfrost.so
%{_datadir}/vulkan/icd.d/panfrost_icd.*.json
%endif

%files vulkan-devel

%files va-drivers
%{_libdir}/dri/nouveau_drv_video.so
%{_libdir}/dri/radeonsi_drv_video.so

%changelog
* Wed May 29 2024 Zhao jiale <zhaojiale@loongson.cn> - 23.1.4-7
- Add upstream orcjit patch and support loongarch64 orcjit

* Wed Jan 17 2024 Rebuild Robot <rebot@opencloudos.org> - 23.1.4-6
- Rebuilt for elfutils

* Mon Dec 25 2023 luffyluo <luffyluo@tencent.com> - 23.1.4-5
- Rebuilt for llvm 17.0.6

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 23.1.4-4
- Rebuilt for OpenCloudOS Stream 23.09

* Tue Aug 22 2023 Wang Guodong <gordonwwang@tencent.com> - 23.1.4-3
- Rebuilt for libva 2.19.0

* Thu Aug 17 2023 Shuo Wang <abushwang@tencent.com> - 23.1.4-2
- Rebuilt for vulkan-loader 1.3.250.0

* Wed Aug 02 2023 cunshunxia <cunshunxia@tencent.com> - 23.1.4-1
- upgrade to 23.1.4

* Fri Jul 14 2023 Xiaojie Chen <jackxjchen@tencent.com> - 23.1.3-1
- Upgrade to upstream version 23.1.3

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 22.2.3-5
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 22.2.3-4
- Rebuilt for OpenCloudOS Stream 23

* Mon Jan 09 2023 cunshunxia <cunshunxia@tencent.com> - 22.2.3-3
- fix build error on arm.

* Fri Dec 09 2022 Xiaojie Chen <jackxjchen@tencent.com> - 22.2.3-2
- Rebuild mesa

* Wed Nov 16 2022 cunshunxia <cunshunxia@tencent.com> - 22.2.3-1
- initial build
